#define the names of our sets
set CROPS;                   # all crops
set REQ_CROPS within CROPS;  # crops with minimum requirements, subset of CROPS
set QUOTA_CROPS within CROPS;# crops with quota, subset of CROPS
set SCEN;                    #scenarios

# declare our parameters in the problem
param prob{SCEN}>=0;

param max_land >0;

param yield {CROPS,SCEN};
param selling_price{CROPS,SCEN};
param planting_cost{CROPS};

param penalty{QUOTA_CROPS};
param quota{QUOTA_CROPS};

param req_amount{REQ_CROPS};
param purch_price{REQ_CROPS,SCEN};

# declare our decision variables (all are positive)
var farming{CROPS} >=0;
var selling{CROPS,SCEN}>=0;
var buying{REQ_CROPS,SCEN}>=0;
var selling_at_penalty{QUOTA_CROPS,SCEN}>=0;



# We minimize the average taken over all scenarios of the income of selling,
# minus costs for buying, planting and quota penaltys.

maximize profit :sum{s in SCEN} prob[s]*(
         sum{i in CROPS} selling[i,s]*selling_price[i,s]
         - sum{i in REQ_CROPS} buying[i,s]*purch_price[i,s]
         - sum{i in QUOTA_CROPS} penalty[i]*selling_at_penalty[i,s]
         - sum{i in CROPS} planting_cost[i]*farming[i]);         


# make sure we pay a penalty for all beets sold above the quota
# note that the penalty of 26$ per ton gives a selling price of 10$ per ton
# above the quota.
subject to  Quotas {i in QUOTA_CROPS,s in SCEN}: selling[i,s]
                            <=quota[i]+ selling_at_penalty[i,s];


# make sure we got enough of the required crops
subject to Required {i in REQ_CROPS, s in SCEN} : 
                       farming[i]*yield[i,s] - selling[i,s]
                        +buying[i,s] >=req_amount[i];

# Make sure we do not sell more than we have of crops without requirements
subject to Mass_balance {i in CROPS diff REQ_CROPS, s in SCEN}: 
                       farming[i]*yield[i,s] >= selling[i,s];

# make sure we do not plant too much.
subject to Area: sum{i in CROPS} farming[i] <= max_land;
 
